<?php
namespace SGalinski\SgYoutube\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Helper functions
 */
class YoutubeService {
	const API_KEY = 'AIzaSyCDgiB2v8gW00WFo4z0H99XWJJiJRVdnlc';
	const API_URL = 'https://www.googleapis.com/youtube/v3/';
	const API_CHANNEL = 'search';
	const API_PLAYLIST = 'playlistItems';
	const API_PART = 'snippet';
	const API_ORDER_BY = 'date';

	/**
	 * @param string $youtubeId
	 * @param string $maxResults
	 * @return string
	 */
	public function getApiUrl($youtubeId = '', $maxResults = '10') {
		$apiUrl = self::API_URL;
		$parameters = [];

		if (preg_match('#^UC#', $youtubeId)) {
			$apiUrl .= self::API_CHANNEL;
			$parameters['channelId'] = $youtubeId;
		} else if (preg_match('#^PL#', $youtubeId)) {
			$apiUrl .= self::API_PLAYLIST;
			$parameters['playlistId'] =  $youtubeId;
		} else {
			throw new \InvalidArgumentException(
				'Given ID is neither a channel nor a playlist.', 403
			);
		}

		$parameters['order'] = self::API_ORDER_BY;
		$parameters['part'] = self::API_PART;
		$parameters['key'] = self::API_KEY;
		$parameters['maxResults'] = $maxResults;
		$query = http_build_query($parameters);

		return $apiUrl . '?' . $query;
	}

	/**
	 * @param array $jsonArray
	 * @param string $youtubeId
	 * @return array
	 */
	public function mapArray($jsonArray = [], $youtubeId = '') {
		// TODO: resolve redundancy of conditional
		if (preg_match('#^UC#', $youtubeId)) {
			return array_map(
				function ($field) {
					return [
						'title' => $field['snippet']['title'],
						'description' => $field['snippet']['title'],
						'thumbnail' => $field['snippet']['thumbnails']['high']['url'],
						'url' => 'https://www.youtube.com/watch?v=' . $field['id']['videoId']
					];
				}, $jsonArray
			);
		} else if (preg_match('#^PL#', $youtubeId)) {
			return array_map(
				function ($field) {
					return [
						'title' => $field['snippet']['title'],
						'description' => $field['snippet']['title'],
						'thumbnail' => $field['snippet']['thumbnails']['high']['url'],
						'url' => 'https://www.youtube.com/watch?v=' . $field['snippet']['resourceId']['videoId']
					];
				}, $jsonArray
			);
		} else {
			throw new \InvalidArgumentException(
				'Given ID is neither a channel nor a playlist.', 403
			);
		}
	}


	/**
	 * @param string $youtubeId
	 * @param string $maxResults
	 * @return array|mixed
	 */
	public function getJsonAsArray($youtubeId = '', $maxResults = '10') {
		$url = $this->getApiUrl($youtubeId, $maxResults);
		$jsonString = '';
		if (function_exists('curl_init')) {
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, TRUE);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt($ch, CURLOPT_URL, $url);
			$jsonString = curl_exec($ch);
			curl_close($ch);

		} elseif (ini_get('allow_url_fopen')) {
			$jsonString = file_get_contents($url);
		} else {
			throw new \InvalidArgumentException(
				'The system doesn\'t have CURL enabled and "allow_url_fopen" is disabled.', 403
			);
		}

		$jsonArray = ($jsonString !== '' ? json_decode($jsonString, TRUE) : []);
		if ($jsonArray === NULL) {
			throw new \InvalidArgumentException(
				'There is something wrong with loaded JSON or encoded data is deeper than the recursion limit.', 403
			);
		}

		if (!$jsonArray) {
			throw new \InvalidArgumentException('JSON could\'t be parsed.', 403);
		}

		//By API documentation provided 'error' key is sent if, probably, URL can not return JSON data or Permission is denied.
		if (isset($jsonArray['error'])) {
			throw new \InvalidArgumentException('Message: ' . $jsonArray['error'], 403);
		}

		if (!isset($jsonArray['items'])) {
			throw new \InvalidArgumentException('No items array.', 403);
		}

		return $this->mapArray($jsonArray['items'], $youtubeId);
	}

}
